@@ -109,9 +109,9 @@ xasprintf (char **str, const char *fmt, ...)
 }
 
 int
-write_file_at (int dirfd, const char *name, const void *data, size_t len, libcrun_error_t *err)
+write_file_at_with_flags (int dirfd, int flags, mode_t mode, const char *name, const void *data, size_t len, libcrun_error_t *err)
 {
-  cleanup_close int fd = openat (dirfd, name, O_WRONLY | O_CREAT | O_TRUNC, 0700);
+  cleanup_close int fd = openat (dirfd, name, O_WRONLY | flags, mode);
   int ret = 0;
   if (UNLIKELY (fd < 0))
     return crun_make_error (err, errno, "opening file `%s` for writing", name);
@@ -126,6 +126,12 @@ write_file_at (int dirfd, const char *name, const void *data, size_t len, libcru
   return ret;
 }
 
+int
+write_file_at (int dirfd, const char *name, const void *data, size_t len, libcrun_error_t *err)
+{
+  return write_file_at_with_flags (dirfd, O_CREAT | O_TRUNC, 0700, name, data, len, err);
+}
+
 int
 write_file_with_flags (const char *name, int flags, const void *data, size_t len, libcrun_error_t *err)
 {
@@ -385,9 +391,12 @@ safe_openat (int dirfd, const char *rootfs, size_t rootfs_len, const char *path,
 
   if (openat2_supported)
     {
-      ret = TEMP_FAILURE_RETRY (syscall_openat2 (dirfd, path, flags, mode, RESOLVE_IN_ROOT));
-      if (ret < 0)
+    repeat:
+      ret = syscall_openat2 (dirfd, path, flags, mode, RESOLVE_IN_ROOT);
+      if (UNLIKELY (ret < 0))
         {
+          if (errno == EINTR || errno == EAGAIN)
+            goto repeat;
           if (errno == ENOSYS)
             openat2_supported = false;
           if (errno == ENOSYS || errno == EINVAL || errno == EPERM)
@@ -451,13 +460,11 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
 
   path = consume_slashes (path);
 
-  npath = xstrdup (path);
+  /* Empty path, nothing to do.  */
+  if (*path == '\0')
+    return 0;
 
-  it = npath + strlen (npath) - 1;
-  while (*it == '/' && it > npath && ((size_t) (it - path)) > dirpath_len)
-    *it-- = '\0';
-  if (((size_t) (it - path)) == dirpath_len)
-    return crun_make_error (err, 0, "invalid path `%s`", path);
+  npath = xstrdup (path);
 
   cwd = dirfd;
   cur = npath;
@@ -547,7 +554,12 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
   if (do_open)
     {
       if (cwd == dirfd)
-        return dup (dirfd);
+        {
+          ret = dup (dirfd);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, errno, "dup `%s`", dirpath);
+          return ret;
+        }
 
       wd_cleanup = -1;
       return cwd;
@@ -607,10 +619,8 @@ crun_ensure_file_at (int dirfd, const char *path, int mode, bool nofollow, libcr
       if (UNLIKELY (ret < 0))
         return ret;
       *it = '/';
-
-      return create_file_if_missing_at (dirfd, tmp, err);
     }
-  return 0;
+  return create_file_if_missing_at (dirfd, tmp, err);
 }
 
 int
@@ -1498,7 +1508,7 @@ run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, i
       char *tmp_args[] = { path, NULL };
       int dev_null_fd = -1;
 
-      ret = mark_for_close_fds_ge_than (3, err);
+      ret = mark_or_close_fds_ge_than (3, false, err);
       if (UNLIKELY (ret < 0))
         libcrun_fail_with_error ((*err)->status, "%s", (*err)->msg);
 
@@ -1536,7 +1546,7 @@ run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, i
 }
 
 int
-mark_for_close_fds_ge_than (int n, libcrun_error_t *err)
+mark_or_close_fds_ge_than (int n, bool close_now, libcrun_error_t *err)
 {
   cleanup_close int cfd = -1;
   cleanup_dir DIR *dir = NULL;
@@ -1545,7 +1555,7 @@ mark_for_close_fds_ge_than (int n, libcrun_error_t *err)
   struct statfs sfs;
   struct dirent *next;
 
-  ret = syscall_close_range (n, UINT_MAX, CLOSE_RANGE_CLOEXEC);
+  ret = syscall_close_range (n, UINT_MAX, close_now ? 0 : CLOSE_RANGE_CLOEXEC);
   if (ret == 0)
     return 0;
   if (ret < 0 && errno != EINVAL && errno != ENOSYS && errno != EPERM)
@@ -1581,15 +1591,24 @@ mark_for_close_fds_ge_than (int n, libcrun_error_t *err)
       if (val < n || val == fd)
         continue;
 
-      ret = fcntl (val, F_SETFD, FD_CLOEXEC);
-      if (UNLIKELY (ret < 0))
-        return crun_make_error (err, errno, "cannot set CLOEXEC fd for '/proc/self/fd/%s'", name);
+      if (close_now)
+        {
+          ret = close (val);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, errno, "close(fd=%d)", val);
+        }
+      else
+        {
+          ret = fcntl (val, F_SETFD, FD_CLOEXEC);
+          if (UNLIKELY (ret < 0))
+            return crun_make_error (err, errno, "cannot set CLOEXEC fd for '/proc/self/fd/%s'", name);
+        }
     }
   return 0;
 }
 
 void
-get_current_timestamp (char *out)
+get_current_timestamp (char *out, size_t len)
 {
   struct timeval tv;
   struct tm now;
@@ -1599,7 +1618,8 @@ get_current_timestamp (char *out)
   gmtime_r (&tv.tv_sec, &now);
   strftime (timestamp, sizeof (timestamp), "%Y-%m-%dT%H:%M:%S", &now);
 
-  sprintf (out, "%s.%09ldZ", timestamp, tv.tv_usec);
+  snprintf (out, len, "%s.%09ldZ", timestamp, tv.tv_usec);
+  out[len - 1] = '\0';
 }
 
 int
@@ -1663,8 +1683,8 @@ check_access (const char *path)
   return 0;
 }
 
-const char *
-find_executable (const char *executable_path, const char *cwd, const char *handler)
+char *
+find_executable (const char *executable_path, const char *cwd)
 {
   cleanup_free char *cwd_executable_path = NULL;
   cleanup_free char *tmp = NULL;
@@ -1698,11 +1718,6 @@ find_executable (const char *executable_path, const char *cwd, const char *handl
       executable_path = cwd_executable_path;
     }
 
-  /* Process executable according to configured handler */
-  /* For `wasm` we don't want to check access hence return early*/
-  if (handler != NULL && (strcmp (handler, "wasm") == 0))
-    return xstrdup (executable_path);
-
   /* Absolute path.  It doesn't need to lookup $PATH.  */
   if (executable_path[0] == '/')
     {
@@ -2006,20 +2021,28 @@ copy_recursive_fd_to_fd (int srcdirfd, int dfd, const char *srcname, const char
 }
 
 const char *
-find_annotation (libcrun_container_t *container, const char *name)
+find_annotation_map (json_map_string_string *annotations, const char *name)
 {
   size_t i;
 
+  if (annotations == NULL)
+    return NULL;
+
+  for (i = 0; i < annotations->len; i++)
+    {
+      if (strcmp (annotations->keys[i], name) == 0)
+        return annotations->values[i];
+    }
+  return NULL;
+}
+
+const char *
+find_annotation (libcrun_container_t *container, const char *name)
+{
   if (container->container_def->annotations == NULL)
     return NULL;
 
-  for (i = 0; i < container->container_def->annotations->len; i++)
-    {
-      if (strcmp (container->container_def->annotations->keys[i], name) == 0)
-        return container->container_def->annotations->values[i];
-    }
-
-  return NULL;
+  return find_annotation_map (container->container_def->annotations, name);
 }
 
 ssize_t
